#!/opt/misc/bin/perl
# #!/bin/sh
                                    $group4::version = "hks v1.007 11 Apr 00";
#-----------------------------------------------------------------------------
#
#
#                    GGGGG                                       4
#  H    H   K    K  G     G   RRRRR     OOOO    U    U   PPPPP   4    4
#  H    H   K   K   G         R    R   O    O   U    U   P    P  4    4
#  HHHHHH   KKKK    G  GGGG   R    R   O    O   U    U   P    P  4    4
#  H    H   K  K    G     G   RRRRR    O    O   U    U   PPPPP   4444444
#  H    H   K   K   G     G   R   R    O    O   U    U   P            4
#  H    H   K    K   GGGGG    R    R    OOOO     UUUU    P            4
#
#
#-----------------------------------------------------------------------------
# 1.003 28 Mar 00 dbb Since v16819 & v16817 are currently installed in G/C,
#                     hks must be sure to upload to upper only after a
#                     partial version has move the rx/tx buffers.
#                     This hks does some smart uploading...
# 1.005  7 Apr 00 dbb Added HV tests, Reset Tests, Verbose switch
# 1.006  9 Apr 00 dbb Created this module that just performs the reset tests.
#                     Improved commandedWDR to accept parameter specifying
#                     expected diagnostic for the reset performed.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# The 'use strict' directive forces the coder to follow rules that might keep
# him out of trouble by calling attention to undeclared variables and the like.
use strict;
use diagnostics -verbose;

#-----------------------------------------------------------------------------
# This subroutine defines the general method for performing a reset: clear
# diags, print any debug or report messages, do whatever you do to force a
# reset, then verify the resulting diagnostic code is as expected.  Finally,
# print out any debug or report messages, such as the current version info.
#
# Aside from a commanded WDR, there are several other ways to effect a reset.
# This test allows the user to specify an additional parameter that selects
# alternate reset methods:
#   0 - commanded WDR (default)
#   1 - ten-second timeout
#   2 - commanded goto reset vector address 0
#   3 - commanded POR
#   4 - commanded special POR
#
# This is a specific test procedure that verifies correct and expected
# behavior.  Callers may treat this subroutine as a function that
# returns a zero if the test passes, else returns nonzero if test fails.
#-----------------------------------------------------------------------------
sub commandedWDR {
  my $expecteddiag = shift(@_);        # fetch subroutine parameter 0..2Eh
  my $resetmethod = shift(@_);         # fetch subroutine parameter 0..4
  $main::TestCounter++;                # tally that this is a test, and failure
                                       # is a possibility.
  if ($main::Verbose) {
    print "Reset subtest number $main::TestCounter.  ";
    if ($resetmethod) {
      print "Reset method specified is by ";
      if    ($resetmethod == 1) { print "timing out ten seconds"; }
      elsif ($resetmethod == 2) { print "commanded goto reset vector address 0"; }
      elsif ($resetmethod == 3) { print "commanded POR"; }
      elsif ($resetmethod == 4) { print "commanded special POR"; }
      else                      { print "Unknown!"; }
    } # if
    else                        { print "commanded WDR"; }
    print "\n";
  } # if verbose

  my $result = removediags; # clear list so we can note updates
  if ($result) {
    print "Unable to clear diagnostic stack!  ";
    return 99; # assume an error
  } # if 

  if ($resetmethod) {
    if    ($resetmethod == 1) { sleep 10; } # timing out ten seconds
    elsif ($resetmethod == 2) { sendcommand(0xeaea,0,0,0); } # goto 0
    elsif ($resetmethod == 3) { sendcommand(0xf0f0,0,0,0); } # POR
    elsif ($resetmethod == 4) { system("cmd_client 0x80000000 > /dev/null"); }
    else                      { print "Unknown!"; }
  } # if
  else                        { sendcommand(0xf5f5,0,0,0); } #RSTS

  # now wait for activity at counters.
  $result = checkactivity;
  print "CheckActivity: $result\n";

  if (defined $main::Verbose) { if ($main::Verbose > 1024) {
    print "checking diags... ";
  }} # if verbose
  $result = getdiag0; # s/b 1d for v16819
  if ($result != $expecteddiag) {
    $main::Failures++;
    printf "Expected diag %x: ", $expecteddiag;
    printf ":got %x:", $result;
    print "*Fail*";
  } # if
  else {
    $result = 0; # Pass!
    print "Pass";
  }

  if (defined $main::Verbose) { if ($main::Verbose > 8192) {
    print "a little sleepy...  ";
  }} # if verbose
  system("sleep 1");

  if (defined $main::HexUploadFilename) {
    sendcommand(0xeaea, 0xfe01, 0, 0); # goto upper reentry
    if (defined $main::Verbose) { if ($main::Verbose > 8192) {
      print "a little sleepy...  ";
    }} # if verbose
    system("sleep 1");
  } # if 
  showversioninfo;
  return $result;
} # sub commandedWDR

#-----------------------------------------------------------------------------
sub group04tests {
  my $result = 0;
  my $i = 0;
  useversion; # make sure the specified operate image is executing,
              # default is boot image.
  TestHeader("Test 4 -- Reset");
  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    The Reset Tests demonstrate the various reset modes of the DIB/DCE
    software.  The following reset tests are performed as described below:
      * commanded reset 
      * watchdog timeout -- watchdog enabled
      * watchdog timeout -- watchdog disabled
      * commanded POR
      * special POR command
      * -- not tested! external reset --
      * POR after 256 commanded resets 

    Reset tests are generally performed by clearing the diagnostic stack, enabling
    or disabling the watchdog as required, then performing some act the should result
    in a reset and checking for expected performance in diagnostics and timers.  Such
    acts include commanding a WDR (soft) or POR (hard) or special reset command or
    simply sending nothing for ten seconds.

    The sequence of the subtests is arbitrary, and some tests may be disabled for the
    user's convenience.\n";
  }} # if verbose output requested

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    Special Reset Test -- ";
  }} # if verbose output requested
  $result = commandedWDR(0x1b,4); # command a special POR and wait
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    Watchdog Timeout Test -- determine that watchdog timeout does not occur.  ";
  }} # if verbose output requested
  $result = commandedWDR(0x0,1); # command a timeout and wait
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    Commanded Reset Test -- ";
  }} # if verbose output requested
  $result = commandedWDR(0x1c,0); # command a WDR and wait
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    Watchdog Timeout Test -- determine that watchdog timeout does occur.  ";
  }} # if verbose output requested
  $result = commandedWDR(0x1c,1); # command a timeout and wait
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    Reset Vector Test -- ";
  }} # if verbose output requested
  $result = commandedWDR(0x1c,2); # command a goto reset vector
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n    POR Reset Test -- ";
  }} # if verbose output requested
  $result = commandedWDR(0x1b,3); # command a POR and wait
  if ($result) { return $result; } # abort on failure.

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "    The previous test left us in the POR reset state, so we know the reset counter
    is zero.  A requirement of the initialization routine is to force a POR reset after 256
    WDR resets.  The test performs 256 resets and passes only when 255 diag 1C appears and 
    on the 256th reset a diag 1B appears.";
  }} # if verbose output requested

  for ($i = 1; (($i < 257) && (!$result)); $i++) {
    print "Subtest $i: ";
    $result = commandedWDR(0x1c); # command a WDR and wait
  } # for
  if (!$result) {
    $result = commandedWDR(0x1b); # command a WDR and wait
  } # if

  if (defined $main::Verbose) { if ($main::Verbose) {
    print "\n\n    Reset Tests Complete!\n\007";
  }} # if verbose output requested
  return; # this is a quick way to abort this subroutine here.

} # sub group04tests
