#!/opt/misc/bin/perl
# #!/bin/sh
                                      $main::version = "hks v1.011 17 Apr 00";
# The brief editor will update this line     Last update::  4-20-100 1:51 pm 
#-----------------------------------------------------------------------------
#
#                    GGGGG                                        22222
#  H    H   K    K  G     G   RRRRR     OOOO    U    U   PPPPP   2     2
#  H    H   K   K   G         R    R   O    O   U    U   P    P        2
#  HHHHHH   KKKK    G  GGGG   R    R   O    O   U    U   P    P   22222
#  H    H   K  K    G     G   RRRRR    O    O   U    U   PPPPP   2
#  H    H   K   K   G     G   R   R    O    O   U    U   P       2
#  H    H   K    K   GGGGG    R    R    OOOO     UUUU    P       2222222
#
#-----------------------------------------------------------------------------
#                                      Last update::  4-13-100 8:44 am 
# 1.003 28 Mar 00 dbb Since v16819 & v16817 are currently installed in G/C,
#                     hks must be sure to upload to upper only after a
#                     partial version has move the rx/tx buffers.
#                     This hks does some smart uploading...
# 1.005  7 Apr 00 dbb Added HV tests, Reset Tests, Verbose switch
# 1.006 10 Apr 00 dbb Fixed a bug in cmd, reorganized hks tool into a
#                     collection of .pm files.
# 1.007 13 Apr 00 dbb Sorted collateral lists in preparation for redesign
#                     that assembles lists as needed, instead of explicitly
#                     defining each list.  Should make it clearer and terse.
# 1.010 17 Apr 00 dbb Evolving design to command scripts to this:
#                      A command script is a list of commands that includes
#                      the following information:  the command opcode,
#                      the talkback address, a bit mask, a diag code, a
#                      reference to a list of required talkback values,
#                      a target setting, and a reference to a list of allowed
#                      talkback values.  The information about the command
#                      specifies the expected new state of the detector.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# The 'use strict' directive forces the coder to follow rules that might keep
# him out of trouble by calling attention to undeclared variables and the like.
use strict;
use diagnostics -verbose;

#-----------------------------------------------------------------------------
# Test Plan
#  1.  Reset 
#  2.  HV 
#  3.  Digitizer/TDC 
#  4.  Door
#  5.  Special Commands
#  6.  Operate
#  7.  Sensors
#  
use hkinfo;
use hksubs;
use hkgroups;
use hkTDC;

#-----------------------------------------------------------------------------
# returns 0 if okay, non-zero on test failure
# Compares 16-bit values according to a mask.  If the mask specified all bits,
#  or if the mask specifies only upper or lower byte, then only those bytes
#  are used to compare against the precision limit that allows for dac converts.
#  Else you specify bits that match only those bits in result, set iff param > 0
sub checktb {
  my $actualresult   = shift @_; # 16 bit value from hkprint
  my $expectedresult = shift @_; # 16 bit value from hkprint
  my $tbinfo         = shift @_; # array of bitmap, precision
  my $bitmap         = $tbinfo->[0]; # mask indicates type of comparison
  my $precision      = $tbinfo->[1]; # allowable error
  my $tflag = 0;

  # printf " %04X %04X %04X %04X ", $actualresult, $expectedresult, $bitmap, $precision;

  # Now look at bitmap to determine what the result should look like
  if ($bitmap == 0xffff) {
    if (abs($actualresult - $expectedresult) > $precision) { $tflag = 2; }
  } elsif (($bitmap & 0x0FF) == 0x0FF) { # low
    $actualresult &= 0xFF; # mask off upper bits
    $expectedresult &= 0xFF; # mask off upper bit 
    if (abs($actualresult - $expectedresult) > $precision) { $tflag = 3; }
  } elsif (($bitmap & 0xFF00) == 0xFF00) { # high
    $actualresult = int($actualresult / 256); # shift upper byte to lower
    $expectedresult = int($expectedresult / 256); # mask off upper bit 
    if (abs($actualresult - $expectedresult) > $precision) { $tflag = 4; }
  } else {
    $actualresult &= $bitmap;
    $expectedresult = ($expectedresult)?$bitmap:0;
    if ($actualresult != $expectedresult) { $tflag = 5; }
  } # or bitmapped
  printf " expecting %04X:", $expectedresult;
  printf "got %04X: ", $actualresult;
  # print "*$tflag*" if $tflag;
  return $tflag;
} # sub checktb

#-----------------------------------------------------------------------------
# This is the core test for the hks tool.
# It defines the kind of test done on each item.
# Generally the test involves sending a command,
# then looking at the housekeeping packet
# to determine if the right thing happened.
# Passing test returns nonzero; return zero means some fainure
sub cmdtest12 {

# The test procedure gets a reference to an array of information about the command being tested.
# In addition to the command opcode, the array contains elements that hold the key to a talkback,
# the expected diagnostic, starting and ending values to send as parameters, and a reference to
# a list of talkbacks that are expected to change with/without the command.
  my $OpcodeInfo = shift(@_);            # reference to array of info about test command
  my $opcode = ${$OpcodeInfo}->[0];      # fetch the opcode
  my $param0 = 0;                        # init to prevent uninit warning
  my $readback  = shift(@_);             # fetch list of talkbacks to ignore
  my $expectedresult;                    # filled in from referenced array below
  my $tmpcode = $opcode & 0x00FF;        # lop-off upper byte of opcode so we can
  my $testresult = 0;                    # presumed pass
  $opcode = $tmpcode + (256 * $tmpcode); # construct the proper format for opcodes.

  if (!defined %main::olditems) { 
    %main::olditems = %main::memitems;        # copy the hkp snapshot to local image
  } # if 

  $main::TestCounter++;                  # tally this command test

  $param0 = ${$OpcodeInfo}->[3];
  my %rhash = %{${${$OpcodeInfo}}[2]};   # Required hkp list

  my $timeout = $#$readback; # simply true, inconclusive is not a result of this test
  if (!(defined $hkinfo::CmdNames{$opcode})) {
    $hkinfo::CmdNames{$opcode} = "anonymous";
  } # if it was undefined, define it! and give it a temp name 
  printf "Testing %s %04x $param0 ",  $hkinfo::CmdNames{$opcode}, $opcode;

  if ($timeout) { # if the number of items in readback list is zero, drop out
                  # and don't send command.  Avoid erroneous command.
    $timeout = sendcommand($opcode, $param0, 0, 0); # trial command
    if ($timeout) {                      # true when talkbacks recvd
      my $passctr = 0;
      my $k;
      $timeout = 0; # failure counter for this test
      my $errordetected = 0;
      for $k (0..$#$readback) {
        my $actualresult = hex($main::memitems{$readback->[$k]}); # get 
        my $expectedresult = hex($main::olditems{$readback->[$k]}); # expected result
        if (defined $rhash{$readback->[$k]}) { # check to see if hkp item
                                               # is in list of required results.
          my $bitmap = $rhash{$readback->[$k]}->[0]; # use specified bitmap
          if (!(defined $hkinfo::HkpNames{$readback->[$k]}[0])) {
            $hkinfo::HkpNames{$readback->[$k]}[0] = "anonymous";
          } # if it was undefined, define it! and give it a temp name 
          print "\n At this $readback->[$k] $hkinfo::HkpNames{$readback->[$k]}[0]";
          # a design bug, patched here by shifting the expected value left 1 byte, as required.
          if ($bitmap == 0xFF00) { $param0 *= 256; }
          printf " was %04X", $expectedresult;
          $timeout = checktb($actualresult, $param0, $rhash{$readback->[$k]});

          if (!$timeout) {
            # Now we have an addition hurdle to clear: is the diagnostic as expected?
            my $diag0 = getdiag0;
            if (${$OpcodeInfo}->[1] == $diag0) {
              if ($actualresult == $expectedresult) {
                print "Nothing changed! ";
              } else {
                print "Pass";
                $passctr++;
              } # else 
            } else {
              printf "Diagnostic not %x as expected! (%x) *Fail*", ${$OpcodeInfo}->[1], $diag0;
              removediags; # the test involves checking for a new diagnostic, so we clear the old
              $errordetected++; # tally a failure for this test
            }
          } else {
            if ($actualresult == $expectedresult) {
              print "Nothing changed! ";
            } else {
              print "Wrong bits changed!!  *Fail*";
            }
            $errordetected++; # tally a failure for this test
          } # else
        } else {
          if (($expectedresult != $actualresult)) {
		    # something changed at a location as expected by OpcodeInfo
          	    # and now we have to decide if the correct thing changed.
          	    # if the mask has FF in it, then we assume the trial param0
          	    # should be readback as is; otherwise the mask specifies the
          	    # bits expected to be cleared if param0 is zero, else set.
            print "\n";
            if (!(defined $hkinfo::HkpNames{$readback->[$k]}[0])) {
              $hkinfo::HkpNames{$readback->[$k]}[0] = "anonymous";
            } # if it was undefined, define it! and give it a temp name 

              # til we figure out what to do...
            printf " %04X $param0 At $readback->[$k] <%s>", $opcode, $hkinfo::HkpNames{$readback->[$k]}[0];
            printf " was %04X:", $expectedresult;
            printf "got %04X:", $actualresult;

              # At this point we have detected a change in housekeeping and we want to know if
              #  the housekeeping item is on the list of acceptable changes.  So we loop thru
              #  the list of acceoptable changes and flag an error only if the list is empty,
              #  or if no items on list are found, or if the item found is out of bounds.
            my %thash = %{${${$OpcodeInfo}}[4]}; # Allowed hkp list
            my $tflag = 0; # assume in error
            if (defined %thash) {
              my $j; my $FoundAtLeastOne = 0;
              for $j (keys %thash) {
                if ($j eq $readback->[$k]) {
                  # we should check that the mask bits are right -- later! dbb 4-5-00
                  my $bitmap = $thash{$j}->[0];
                     # This talkback should be close to the previous value, not the command parameter
                  my $result = checktb($actualresult, $expectedresult, $thash{$readback->[$k]});
                  if ($result) { $tflag++; }
                  $FoundAtLeastOne++; 
                } 
              } # for
              if (!$FoundAtLeastOne) { $tflag++; }
            } # if defined thash
            if (!$tflag) {
              print "Okay...";
            } else {
              print "Wrong location changed! *Fail*";
              $errordetected++; # tally a failure for this test
            } # else
          } # not what we were expecting...
        } # else
      } # for
      %main::olditems = %main::memitems;        # copy the hkp snapshot to local image
      print "\nTest $main::TestCounter Result: "; 

      if ($errordetected ) {
        print "*Fail*";
        print "\t\t\t          : ", scalar(localtime), "\n";
        $main::Failures++;
        $timeout = 0; # tally a failure for this test
        return 1;
      } else { 
        # Now we have an addition hurdle to clear: is the diagnostic as expected?
        my $diag0 = getdiag0;
        if (${$OpcodeInfo}->[1] == $diag0) {
          if (!$passctr) { 
            print "Inconclusive"; # Wrong state for testing!
            $main::Inconclusives++;
          } # No positive response from detector, possibly not a failure.
          else {
            print "Pass";
          } # else
          $timeout = 99; # we're calling it a non-failure, and reporting it that way
          print "\t\t\t          : ", scalar(localtime), "\n";
        } else {
          printf "Diagnostic not %x as expected! (%x) *Fail*", ${$OpcodeInfo}->[1], $diag0;
          removediags; # the test involves checking for a new diagnostic, so we clear the old
          $timeout = 0; # tally a failure for this test
          # $timeout++; # tally a failure for this test
          print "\t\t\t          : ", scalar(localtime), "\n";
          $main::Failures++;
          return 2;
        }
      } # else
    } else {
      print "*Timeout*";
      print "\t\t\t          : ", scalar(localtime), "\n";
      $main::Failures++;
      return 3;
    } # no response is a failure.
  } # if the bits are in the right state for testing.
  else {
    print "Inconclusive"; # Wrong state for testing!
    print "\t\t\t          : ", scalar(localtime), "\n";
    $main::Inconclusives++;
    $timeout = 99; # we're calling it a non-failure, and reporting it that way
  } # else
  print "\n";
  if (!$timeout) { $testresult = 4; } # when timeout is zero, a failure occurred.
  return $testresult; # pass is zero
} # sub cmdtest12

#-----------------------------------------------------------------------------
sub ProcessTestSet {

  my $TestSet = (shift(@_)); # a reference to the list of command specifiers
  my @CheckList;             # list of items that are interesting when changed
  my @hkpitems = keys %main::memitems; # start with the set of all hkp items
  my $i; my $j; my $k;       # local variables
  my %zapplist = split(" ",`hkprint 0x1040-153e`); # get a list of ph addresses
  my @zaplist = keys %zapplist; # make a list of ph addresses alone
  my $totalf = 0;

  for $k (0..$#hkpitems) {
    my $AddToList = 1; # flag, assume true til proven otherwise
    for $j (0..$#zaplist) {
      if ($hkpitems[$k] eq $zaplist[$j]) { $AddToList = 0; }
    } # for
    for $j (0..$#hkinfo::RemoveFromList) {
      if ($hkpitems[$k] eq $hkinfo::RemoveFromList[$j]) { $AddToList = 0; }
    } # for
    if ($AddToList) {
      push @CheckList, $hkpitems[$k];
    } # if
  } # for

  for $k (0..$#$TestSet) {
    my $result = cmdtest12(\${$TestSet}[$k], \@CheckList);
    if (defined $hkgroup256::fatalfail) {
      if ($hkgroup256::fatalfail) {
        if ($result) { $hkgroup256::fatalfail--; $totalf++; } # continue testing
      } else {
        if ($result) { return $result; } # discontinue testing on first error
      } # else
    } else {
      if ($result) { return $result; } # discontinue testing on first error
    } # fatalfail not defined.
  } # for
  return $totalf;
} # sub ProcessTestSet

#-----------------------------------------------------------------------------
sub group100tests {
  TestHeader("Test 256 -- HV");
  my $HVsetRange = 8; # highest value to send to HV
  my $WigRm = 0x19;
  my @NoopDelay = (
     [ 0x80, 0, { "0x1664" => [ 0x007F, 0] },
                     0, {
                      %hkinfo::HKPbasic,
                  "0x16d4" => [ 0x00FF, 0x30 ], # hv_voltmon
                  "0x16d2" => [ 0x00FF, 14 ], # hv_current monitor change when HV powered on
                                          } ] , # noop, let hvi settle
                  ); # NoopDelay
  my @HVpwron = (
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm],
                  "0x16d4" => [ 0x00FF, 0x40], # hv_voltmon
                  "0x16f2" => [ 0x0020, 0x0020],
                  "0x17d0" => [ 0x00FF, 0x01 ],  # dac setting to 0 when HV powered on
                  "0x1726" => [ 0x00FF, 0x01 ],  # max setting to 0 when HV powered on
                  "0x172a" => [ 0x1818, 0x1818],  # HVPWR, HVENA, MPWR, AUXPWR
                                                }, 1, { %hkinfo::HKPbasic,
             "0x16d2" => [ 0x00FF, 14 ], # hv_current monitor change when HV powered on
                                        } ],                         # HVPWR on
                ); # HVpwron
  my %HVbasic = ( %hkinfo::HKPbasic,
                  "0x16d4" => [ 0x00FF, 0x04], # hv_voltmon
                ); # HVbasic

  @main::HV0TestSet =   ( # HV subtest
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm]
                                                }, 0, \%hkinfo::HKPbasic ], # DIAGC 
     [ 0xce, 0, { "0x1774" => [ 0x00FF, 0] 
                                                }, 7, \%hkinfo::HKPbasic ], # RAMPT 
     [ 0xd1, 0, { "0x16f2" => [ 0x0040, $WigRm] 
                                                }, 1, \%hkinfo::HKPbasic ], # HVENA on
     [ 0xce, 0, { "0x1774" => [ 0x00FF, 0] 
                                                }, 0, \%hkinfo::HKPbasic ], # RAMPT 
     [ 0xdc, 0, { "0x170a" => [ 0x00FF, 0], 
                                                }, 0xCC, \%HVbasic ], # AuxI limit AUXILIM
     @HVpwron,
     @NoopDelay,
     @NoopDelay,
     @NoopDelay,
     @NoopDelay,
     [ 0xcb, 0, { "0x17d0" => [ 0x00FF, 0],
      #            "0x16d4" => [ 0x00FF, 0x50], # hv_voltmon
                                                }, 5, { %hkinfo::HKPbasic,
             "0x1776" => [ 0xffff, 0xffff ], # BITS#: ignore all bits in this talkback
      #      "0x16d2" => [ 0x00FF,  5 ], # hv_current still changing? 
                                        } ],                         # HVDAC
     @NoopDelay,
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, 0],
                                        }, 0x10, { 
                     %hkinfo::HKPbasic,
                  "0x16d4" => [ 0x00FF, 0x0c],
                                        } ],  # HVMAX 16
     @NoopDelay,
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm],
                                                 }, 0x10, {
                     %hkinfo::HKPbasic,
                  "0x17d0" => [ 0x00FF, 0x10 ], # setting to 0 when HV powered on
                  "0x16d4" => [ 0x00FF, 8],
                                        } ],                         # HVDAC 16
     @NoopDelay,
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm],
                  "0x17d0" => [ 0x00FF, 0x0 ],  # set to 0 
                                                 }, 0, { %hkinfo::HKPbasic,
                  "0x17d0" => [ 0x00FF, $HVsetRange ], # setting to 0 when HV powered on
                  "0x16d4" => [ 0x00FF, 0x0a ], # hv_voltmon
                                        } ],                         # HVDAC
     @NoopDelay,
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic],                 # HVMAX
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm],
                  "0x16f2" => [ 0x0020, 0x0000], # bits0
                                                }, 0, { 
                              %HVbasic,
                  "0x16d2" => [ 0x00FF,  0x12 ], # hv_current still changing? 
                  "0x16d4" => [ 0x00FF, 0x30], # hv_voltmon
                                        } ],                         # HVPWR off

     # hv_voltmon takes a long time to settle, so we call 2 nops to stall for time.
     @NoopDelay,
     @NoopDelay,
     @NoopDelay,
     @NoopDelay,
     [ 0xce, 0, { "0x1774" => [ 0x00FF, 0] 
                                                }, 17, \%hkinfo::HKPbasic ], # RAMPT 
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, {
                      %hkinfo::HKPbasic,
                  "0x16d4" => [ 0x00FF, 0x10 ], # hv_voltmon
                                        } ] , # HVENA off
                                          );

  @main::HVXTestSet =    ( # HV cleanup
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, { %hkinfo::HKPbasic,
                                                        } ],            # DIAGC
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 0, { %hkinfo::HKPbasic, 
                                                        } ],            # HVPWR off
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic
                                                          ],            # HVENA off
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 0, { %hkinfo::HKPbasic,
                                                        } ],            # HVPWR off
     [ 0xdc, 0, { "0x170a" => [ 0x00FF, 0], 
                                                }, 0x22, \%hkinfo::HKPbasic ], # AuxI limit AUXILIM
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm],
                  "0x17d0" => [ 0x00FF, $HVsetRange ], # setting to 0 when HV powered on
                                                }, 2, { %hkinfo::HKPbasic, }], # HVDAC
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, $WigRm] }, 3, { %hkinfo::HKPbasic,
             "0x16f2" => [ 0x0020, 0 ], # HVON, HVARM
                                        } ],                         # HVMAX
     [ 0xce, 0, { "0x1774" => [ 0x00FF, 0] 
                                                }, 0, \%hkinfo::HKPbasic ], # RAMPT 
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic ]);               # HVENA off
  @main::HV1TestSet =    ( 
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # DIAGC clear diagnostic codes
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVPWR on
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVPWR off
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # DIAGC clear diagnostic codes
     [ 0xd1, 0, { "0x16f2" => [ 0x0040, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVENA on
     [ 0xd1, 0, { "0x16f2" => [ 0x0040, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVENA on again, diag 9
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # DIAGC clear diagnostic codes
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVENA off
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVENA off again, diag a
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # DIAGC clear diagnostic codes
     [ 0xd1, 0, { "0x16f2" => [ 0x0040, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVENA on
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVPWR on
     [ 0xcb, 0, { "0x1724" => [ 0x00FF, $WigRm] }, 5, \%hkinfo::HKPbasic ],  # HVDAC
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, $WigRm] }, 5, \%hkinfo::HKPbasic ],  # HVMAX
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm] }, 9, \%hkinfo::HKPbasic ],  # HVDAC
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, $WigRm] }, 1, \%hkinfo::HKPbasic ],  # HVMAX
     [ 0xbc, 0, { "0x1722" => [ 0x00FF, $WigRm] }, 260, \%hkinfo::HKPbasic ],  # TOFF
     [ 0xbc, 0, { "0x1722" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # TOFF
     [ 0xb6, 0, { "0x1720" => [ 0x00FF, $WigRm] }, 255, \%hkinfo::HKPbasic ],  # HVLO
     [ 0xba, 0, { "0x171c" => [ 0x00FF, $WigRm] }, 255, \%hkinfo::HKPbasic ],  # PINT
     [ 0xbc, 0, { "0x1722" => [ 0x00FF, $WigRm] }, 260, \%hkinfo::HKPbasic ],  # TOFF
     [ 0xc6, 0, { "0x1624" => [ 0x00FF, $WigRm] }, 55, { %hkinfo::HKPbasic, } ],  # DHVNM
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm] }, 1, { %hkinfo::HKPbasic, } ],  # HVDAC
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic ]); # HVENA off
  @main::HV2TestSet =    ( 
     [ 0xbc, 0, { "0x1722" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # TOFF
     [ 0xf4, 0, { "0x1780" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # DIAGC clear diagnostic codes
     [ 0xcb, 0, { "0x16e2" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVDAC
     [ 0x8d, 0, { "0x1726" => [ 0x00FF, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVMAX
     [ 0xc0, 0, { "0x172c" => [ 0x0001, $WigRm] }, 0, \%hkinfo::HKPbasic ],  # HVPWR off
     [ 0xd1, 0, { "0x16f2" => [ 0x0000, $WigRm] }, 0, \%hkinfo::HKPbasic ]); # HVENA off

  useversion;

  removediags; # the first test involves checking for a diagnostic, so we clear them

  # The first test shows that the hvenable works by requiring it to be set before other hv
  # commands will be accepted.  We send an hvon command and then show that the hv power did not
  # turn on and the expected diagnostic appeared.
  sub SubTestHeader {
    my $textmsg = shift @_;
    print " ------------------ $textmsg -------------------\n";
  } # sub SubTestHeader

  $hkgroup256::fatalfail = 0; # allow one error while executing script
  SubTestHeader("HV subtest");
  ProcessTestSet(\@main::HV0TestSet);
  print "\n";
  if (0) {
  SubTestHeader("HV test");
  ProcessTestSet(\@main::HV1TestSet);
  print "\n";
  SubTestHeader("HV posttest");
  ProcessTestSet(\@main::HV2TestSet);
  print "\n";
  } # if
  $hkgroup256::fatalfail = $#mail::HVXTestSet; # ignore errors while executing script
  SubTestHeader("HV cleanup");
  ProcessTestSet(\@main::HVXTestSet);
  print "\n";
} # group100tests

